/* /////////////////////////////////////////////////////////////////////////////
 * File:        winstl/file_path_buffer.hpp (formerly winstl_file_path_buffer.h)
 *
 * Purpose:     Contains the basic_file_path_buffer template class, and ANSI
 *              and Unicode specialisations thereof.
 *
 * Created:     7th February 2002
 * Updated:     22nd December 2005
 *
 * Thanks to:   Pablo Aguilar for discovering the Borland weirdness which is now
 *              addressed with the calc_path_max_() method.
 *
 * Home:        http://stlsoft.org/
 *
 * Copyright (c) 2002-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/// \file winstl/file_path_buffer.hpp
///
/// Contains the basic_file_path_buffer template class, and ANSI and Unicode specialisations thereof.

#ifndef WINSTL_INCL_WINSTL_HPP_FILE_PATH_BUFFER
#define WINSTL_INCL_WINSTL_HPP_FILE_PATH_BUFFER

#ifndef STLSOFT_DOCUMENTATION_SKIP_SECTION
# define WINSTL_VER_WINSTL_HPP_FILE_PATH_BUFFER_MAJOR       3
# define WINSTL_VER_WINSTL_HPP_FILE_PATH_BUFFER_MINOR       2
# define WINSTL_VER_WINSTL_HPP_FILE_PATH_BUFFER_REVISION    1
# define WINSTL_VER_WINSTL_HPP_FILE_PATH_BUFFER_EDIT        83
#endif /* !STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Compatibility
 */

/*
[Incompatibilies-start]
STLSOFT_COMPILER_IS_MSVC: _MSC_VER<1200
[Incompatibilies-end]
 */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef WINSTL_INCL_WINSTL_H_WINSTL
# include <winstl/winstl.h>
#endif /* !WINSTL_INCL_WINSTL_H_WINSTL */

#if defined(STLSOFT_COMPILER_IS_MSVC) && \
    _MSC_VER < 1100
# error winstl_file_path_buffer.h is not compatible with Visual C++ 5.0 or earlier
#endif /* _MSC_VER < 1200 */

#ifndef STLSOFT_INCL_STLSOFT_HPP_STRING_ACCESS
# include <stlsoft/string_access.hpp>
#endif /* !STLSOFT_INCL_STLSOFT_HPP_STRING_ACCESS */
#ifndef STLSOFT_INCL_STLSOFT_HPP_AUTO_BUFFER
# include <stlsoft/auto_buffer.hpp>
#endif /* !STLSOFT_INCL_STLSOFT_HPP_AUTO_BUFFER */
#ifndef WINSTL_INCL_WINSTL_HPP_PROCESSHEAP_ALLOCATOR
# include <winstl/processheap_allocator.hpp>
#endif /* !WINSTL_INCL_WINSTL_HPP_PROCESSHEAP_ALLOCATOR */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _WINSTL_NO_NAMESPACE
# if defined(_STLSOFT_NO_NAMESPACE) || \
     defined(STLSOFT_DOCUMENTATION_SKIP_SECTION)
/* There is no stlsoft namespace, so must define ::winstl */
namespace winstl
{
# else
/* Define stlsoft::winstl_project */

namespace stlsoft
{

namespace winstl_project
{

# endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup libraries STLSoft Libraries
/// \brief The individual libraries

/// \weakgroup libraries_filesystem File-System Library
/// \ingroup libraries
/// \brief This library provides facilities for defining and manipulating file-system objects

/// \weakgroup winstl_filesystem_library File-System Library (WinSTL)
/// \ingroup WinSTL libraries_filesystem
/// \brief This library provides facilities for defining and manipulating file-system objects for the Win32 API
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

#ifndef STLSOFT_DOCUMENTATION_SKIP_SECTION
template <ss_typename_param_k C>
struct is_ansi
{
    enum { value = 0 };
};

STLSOFT_TEMPLATE_SPECIALISATION
struct is_ansi<ws_char_a_t>
{
    enum { value = 1 };
};
#endif /* !STLSOFT_DOCUMENTATION_SKIP_SECTION */

// class basic_file_path_buffer
/// Acts as a buffer with sufficient size for any drive on the host machine
///
/// This class is a non-template class primarily so that separate instantiations
/// are not created for each instantiation of the basic_file_path_buffer.
///
/// This class provides a simple function, which is to provide the maximum path
/// length for the host. This information is then cached due to the static nature
/// of the get_drivesvar_() method, although it can be reset by calling the
/// refresh() method on the buffer class.
///
/// \param C The character type
/// \param A The allocator type
/// \param CCH The size of the internal member path structure. On translators that support default template arguments this default to (\c 1 + \c _MAX_PATH)
template<   ss_typename_param_k C
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
        ,   ss_typename_param_k A = processheap_allocator<C>
#else /* ? __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT */
        ,   ss_typename_param_k A /* = processheap_allocator<C> */
#endif /* __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT */
        >
class basic_file_path_buffer
{
private:
    enum
    {
        internalBufferSize  =   1 + _MAX_PATH
    };

    enum
    {
            CCH_9x = _MAX_PATH
        ,   CCH_NT = (is_ansi<C>::value ? _MAX_PATH : CONST_NT_MAX_PATH)
    };

    /// The buffer type
    typedef stlsoft_ns_qual(auto_buffer)<   C
                                        ,   A
#if defined(STLSOFT_COMPILER_IS_BORLAND)
    // This is necessary, since Borland will attempt an auto_buffer with what
    // seems like 0 size, or maybe it just can't define the type. Who can tell?
                                        ,   1 + _MAX_PATH
#else /* ? compiler */
                                        ,   internalBufferSize
#endif /* compiler */
                                        >                           buffer_type;
public:
    /// The current parameterisation of the type
    typedef basic_file_path_buffer<C, A>                            class_type;
    /// The value type
    typedef ss_typename_type_k buffer_type::value_type              value_type;
    /// The reference type
    typedef value_type                                              &reference;
    /// The non-mutating (const) reference type
    typedef value_type const                                        &const_reference;
    /// The size type
    typedef ss_typename_type_k buffer_type::size_type               size_type;

// Construction
public:
    /// Default constructor
    basic_file_path_buffer()
        : m_buffer(1 + calc_path_max_())
    {
#ifdef _DEBUG
        memset(&m_buffer[0], '?', m_buffer.size());
        m_buffer[m_buffer.size() - 1] = '\0';
#endif /* _DEBUG */
    }

    basic_file_path_buffer(class_type const &rhs)
        : m_buffer(rhs.size())
    {
        stlsoft_ns_qual(pod_copy_n)(m_buffer.data(), rhs.m_buffer.data(), m_buffer.size());
    }

    class_type &operator =(class_type const &rhs)
    {
        m_buffer.resize(rhs.size());
        stlsoft_ns_qual(pod_copy_n)(m_buffer.data(), rhs.m_buffer.data(), m_buffer.size());

        return *this;
    }

// Operations
public:
    void swap(class_type &rhs) stlsoft_throw_0()
    {
        m_buffer.swap(rhs.m_buffer);
    }

// Accessors
public:
    /// Returns a pointer to a nul-terminated string
    value_type const *c_str() const
    {
        return m_buffer.data();
    }

    /// Returns a mutable (non-const) pointer to the internal buffer
    reference operator [](size_t index)
    {
        buffer_type   &this_  =   m_buffer;

        return this_[index];
    }
#if (   !defined(STLSOFT_COMPILER_IS_COMO) || \
        __COMO_VERSION__ >= 4303) && \
    !defined(STLSOFT_COMPILER_IS_MWERKSx)
    /// Returns a non-mutable (const) pointer to the internal buffer
    const_reference operator [](size_t index) const
    {
        WINSTL_MESSAGE_ASSERT("Index out of range", !(size() < index));

        return m_buffer.data()[index];
    }
#else /* ? compiler */
#endif /* compiler */

    /// Returns the size of the internal buffer
    size_type size() const
    {
        return m_buffer.size();
    }

// Operations
public:
    /// Causes the drives to be examined again for the next instance.
    ///
    /// \deprecated
    static void refresh() stlsoft_throw_0()
    {}

// Implementation
private:
    // Have to do it this way, as an inline, in-MIL tertiary operator causes
    // Borland C++ to think that the answer's 5! 'tis a mysterious beastie,
    // the old Borland compiler, is it not?
    static ws_size_t calc_path_max_()
    {
        ws_size_t   n;

        if(::GetVersion() & 0x80000000)
        {
            n = CCH_9x;
        }
        else
        {
            n = CCH_NT;
        }

        return n;
    }

// Members
private:
    buffer_type m_buffer;
};

/* Typedefs to commonly encountered types. */
/// Instantiation of the basic_file_path_buffer template for the ANSI character type \c char
typedef basic_file_path_buffer<ws_char_a_t, processheap_allocator<ws_char_a_t> >    file_path_buffer_a;
/// Instantiation of the basic_file_path_buffer template for the Unicode character type \c wchar_t
typedef basic_file_path_buffer<ws_char_w_t, processheap_allocator<ws_char_w_t> >    file_path_buffer_w;
/// Instantiation of the basic_file_path_buffer template for the ambeint character type \c TCHAR
typedef basic_file_path_buffer<TCHAR, processheap_allocator<TCHAR> >                file_path_buffer;

/* /////////////////////////////////////////////////////////////////////////////
 * Support for PlatformSTL redefinition by inheritance+namespace, for confused
 * compilers (e.g. VC++ 6)
 */

#ifndef STLSOFT_DOCUMENTATION_SKIP_SECTION

    template<   ss_typename_param_k C
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
            ,   ss_typename_param_k A = winstl_ns_qual(processheap_allocator)<C>
#else /* ? __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT */
            ,   ss_typename_param_k A /* = processheap_allocator<C> */
#endif /* __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT */
            >
    class basic_file_path_buffer__
        : public winstl_ns_qual(basic_file_path_buffer)<C, A>
    {
    private:
        typedef winstl_ns_qual(basic_file_path_buffer)<C, A>            parent_class_type;
    public:
        typedef basic_file_path_buffer__<C, A>                          class_type;
        typedef ss_typename_type_k parent_class_type::value_type        value_type;
        typedef ss_typename_type_k parent_class_type::reference         reference;
        typedef ss_typename_type_k parent_class_type::const_reference   const_reference;
        typedef ss_typename_type_k parent_class_type::size_type         size_type;
    };

#endif /* !STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * swapping
 */

template<   ss_typename_param_k C
        ,   ss_typename_param_k A
        >
inline void swap(basic_file_path_buffer<C, A> &lhs, basic_file_path_buffer<C, A> &rhs)
{
    lhs.swap(rhs);
}

/* /////////////////////////////////////////////////////////////////////////////
 * Shims
 */

template<   ss_typename_param_k C
        ,   ss_typename_param_k A
        >
inline C const *c_str_ptr_null(basic_file_path_buffer<C, A> const &b)
{
    return stlsoft_ns_qual(c_str_ptr_null)(b.c_str());
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k A
        >
inline C const *c_str_ptr(basic_file_path_buffer<C, A> const &b)
{
    return b.c_str();
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k A
        >
inline C const *c_str_data(basic_file_path_buffer<C, A> const &b)
{
    return b.c_str();
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k A
        >
inline ws_size_t c_str_len(basic_file_path_buffer<C, A> const &b)
{
    return stlsoft_ns_qual(c_str_len)(b.c_str());
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k A
        >
inline ws_size_t c_str_size(basic_file_path_buffer<C, A> const &b)
{
    return stlsoft_ns_qual(c_str_size)(b.c_str());
}

template<   ss_typename_param_k S
        ,   ss_typename_param_k C
        ,   ss_typename_param_k A
        >
inline S &operator <<(S & s, basic_file_path_buffer<C, A> const &b)
{
    s << b.c_str();

    return s;
}

/* /////////////////////////////////////////////////////////////////////////////
 * Unit-testing
 */

#ifdef STLSOFT_UNITTEST
# include "./unittest/file_path_buffer_unittest_.h"
#endif /* STLSOFT_UNITTEST */

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group winstl_filesystem_library

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
# if defined(_STLSOFT_NO_NAMESPACE) || \
     defined(STLSOFT_DOCUMENTATION_SKIP_SECTION)
} // namespace winstl
# else
} // namespace winstl_project
} // namespace stlsoft
# endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* In the special case of Intel behaving as VC++ 7.0 or earlier on Win32, we 
 * illegally insert into the std namespace.
 */
#if defined(__STLSOFT_CF_std_NAMESPACE)
# if ( ( defined(STLSOFT_COMPILER_IS_INTEL) && \
         defined(_MSC_VER))) && \
     _MSC_VER < 1310
namespace std
{
    template<   ss_typename_param_k C
            ,   ss_typename_param_k A
            >
    inline void swap(winstl_ns_qual(basic_file_path_buffer)<C, A> &lhs, winstl_ns_qual(basic_file_path_buffer)<C, A> &rhs)
    {
        lhs.swap(rhs);
    }
} // namespace std
# endif /* INTEL && _MSC_VER < 1310 */
#endif /* __STLSOFT_CF_std_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 *
 * The string access shims exist either in the stlsoft namespace, or in the
 * global namespace. This is required by the lookup rules.
 *
 */

#ifndef _WINSTL_NO_NAMESPACE
# if !defined(_STLSOFT_NO_NAMESPACE) && \
     !defined(STLSOFT_DOCUMENTATION_SKIP_SECTION)
namespace stlsoft
{
# else /* ? _STLSOFT_NO_NAMESPACE */
/* There is no stlsoft namespace, so must define in the global namespace */
# endif /* !_STLSOFT_NO_NAMESPACE */

using ::winstl::c_str_ptr_null;

using ::winstl::c_str_ptr;

using ::winstl::c_str_data;

using ::winstl::c_str_len;

using ::winstl::c_str_size;

# if !defined(_STLSOFT_NO_NAMESPACE) && \
     !defined(STLSOFT_DOCUMENTATION_SKIP_SECTION)
} // namespace stlsoft
# else /* ? _STLSOFT_NO_NAMESPACE */
/* There is no stlsoft namespace, so must define in the global namespace */
# endif /* !_STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* WINSTL_INCL_WINSTL_HPP_FILE_PATH_BUFFER */

/* ////////////////////////////////////////////////////////////////////////// */
