/* /////////////////////////////////////////////////////////////////////////////
 * File:        winstl/listview_sequence.hpp (formerly winstl_listview_sequence.h)
 *
 * Purpose:     Contains the listview_sequence class template.
 *
 * Created:     8th May 2003
 * Updated:     22nd December 2005
 *
 * Thanks:      To Pablo Aguilar for making the requisite feature requests.
 *
 * Home:        http://stlsoft.org/
 *
 * Copyright (c) 2003-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/// \file winstl/listview_sequence.hpp
///
/// Contains the listview_sequence class.

#ifndef WINSTL_INCL_WINSTL_HPP_LISTVIEW_SEQUENCE
#define WINSTL_INCL_WINSTL_HPP_LISTVIEW_SEQUENCE

#ifndef STLSOFT_DOCUMENTATION_SKIP_SECTION
# define WINSTL_VER_WINSTL_HPP_LISTVIEW_SEQUENCE_MAJOR      3
# define WINSTL_VER_WINSTL_HPP_LISTVIEW_SEQUENCE_MINOR      1
# define WINSTL_VER_WINSTL_HPP_LISTVIEW_SEQUENCE_REVISION   1
# define WINSTL_VER_WINSTL_HPP_LISTVIEW_SEQUENCE_EDIT       41
#endif /* !STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Compatibility
 */

/*
[Incompatibilies-start]
STLSOFT_COMPILER_IS_BORLAND:
[Incompatibilies-end]
 */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef WINSTL_INCL_WINSTL_H_WINSTL
# include <winstl/winstl.h>
#endif /* !WINSTL_INCL_WINSTL_H_WINSTL */
#ifndef STLSOFT_INCL_STLSOFT_HPP_AUTO_BUFFER
# include <stlsoft/auto_buffer.hpp>
#endif /* !STLSOFT_INCL_STLSOFT_HPP_AUTO_BUFFER */
#ifndef WINSTL_INCL_WINSTL_HPP_PROCESSHEAP_ALLOCATOR
# include <winstl/processheap_allocator.hpp>
#endif /* !WINSTL_INCL_WINSTL_HPP_PROCESSHEAP_ALLOCATOR */
#ifndef STLSOFT_INCL_STLSOFT_HPP_ITERATOR
# include <stlsoft/iterator.hpp>
#endif /* !STLSOFT_INCL_STLSOFT_HPP_ITERATOR */
#ifndef WINSTL_INCL_WINSTL_H_COMMCTRL_FUNCTIONS
# include <winstl/commctrl_functions.h>
#endif /* !WINSTL_INCL_WINSTL_H_COMMCTRL_FUNCTIONS */
#ifndef STLSOFT_INCL_STLSOFT_HPP_SIMPLE_STRING
# include <stlsoft/simple_string.hpp>
#endif /* !STLSOFT_INCL_STLSOFT_HPP_SIMPLE_STRING */
#ifndef STLSOFT_INCL_STLSOFT_HPP_SHIM_STRING
# include <stlsoft/shim_string.hpp>
#endif /* !STLSOFT_INCL_STLSOFT_HPP_SHIM_STRING */
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
typedef stlsoft_ns_qual(basic_simple_string)<TCHAR> lvs_string_t;
#else
typedef stlsoft_ns_qual(basic_simple_string)<   TCHAR
                                            ,   stlsoft_ns_qual(stlsoft_char_traits)<TCHAR>
                                            ,   winstl_ns_qual(processheap_allocator)<TCHAR>
                                            >       lvs_string_t;
#endif /* __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT */
#ifndef WINSTL_INCL_WINSTL_HPP_STRING_ACCESS
# include <winstl/string_access.hpp>        // for string access shims
#endif /* !WINSTL_INCL_WINSTL_HPP_STRING_ACCESS */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _WINSTL_NO_NAMESPACE
# if defined(_STLSOFT_NO_NAMESPACE) || \
     defined(STLSOFT_DOCUMENTATION_SKIP_SECTION)
/* There is no stlsoft namespace, so must define ::winstl */
namespace winstl
{
# else
/* Define stlsoft::winstl_project */

namespace stlsoft
{

namespace winstl_project
{

# endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup libraries STLSoft Libraries
/// \brief The individual libraries

/// \weakgroup libraries_control Control Library
/// \ingroup libraries
/// \brief This library provides facilities for defining and manipulating GUI controls

/// \defgroup winstl_control_library Control Library (WinSTL)
/// \ingroup WinSTL libraries_control
/// \brief This library provides facilities for defining and manipulating Win32 GUI controls
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

/// Item class used by the listview_sequence class
//template<ss_typename_param_k S = lvs_string_t>
class listview_sequence_item
{
public:
//  typedef S           string_type;

public:
    listview_sequence_item(HWND hwndListView, int iIndex)
        : m_hwndListView(hwndListView)
        , m_index(iIndex)
    {}

public:
    lvs_string_t text(int iSubItem = 0) const
    {
        typedef stlsoft_ns_qual(auto_buffer)<   TCHAR
                                            ,   processheap_allocator<TCHAR>
                                            ,   256
                                            >       buffer_t;

        ws_size_t   cb  =   buffer_t::internal_size();
        LV_ITEM     item;

        item.mask       =   LVIF_TEXT;
        item.iItem      =   m_index;
        item.iSubItem   =   iSubItem;

        for(;; cb += buffer_t::internal_size())
        {
            buffer_t    buffer(cb);

            item.cchTextMax =   static_cast<int>(cb);
            item.pszText    =   &buffer[0];

            if(!ListView_GetItem(m_hwndListView, &item))
            {
                item.cchTextMax = 0;
                break;
            }
            else
            {
                ss_size_t len =   static_cast<ss_size_t>(lstrlen(item.pszText));

                if(len + 1 < cb)
                {
                    return lvs_string_t(item.pszText, len);
                }
            }
        }

        return lvs_string_t();
    }

    int             index() const
    {
        return m_index;
    }
    HWND            hwnd() const
    {
        return m_hwndListView;
    }

    int             image() const;
    int             selected_image() const;
    ws_dword_t      data() const
    {
        LV_ITEM item;

        item.mask       =   LVIF_PARAM;
        item.iItem      =   m_index;
        item.iSubItem   =   0;

        return ListView_GetItem(m_hwndListView, &item) ? static_cast<ws_dword_t>(item.lParam) : 0;
    }

private:
    HWND    m_hwndListView;
    int     m_index;
};


/// Provides an STL-like sequence over the contents of a Windows List-view (<code>"SysListView32"</code>)
class listview_sequence
{
public:
    /// The value type
    typedef listview_sequence_item  sequence_value_type;
    typedef sequence_value_type     value_type;
    /// The size type
    typedef ss_size_t               size_type;
    /// The difference type
    typedef ws_ptrdiff_t            difference_type;
    /// 
    typedef listview_sequence       sequence_class_type;


public:
    ss_explicit_k listview_sequence(HWND hwndListView)
        : m_hwndListView(hwndListView)
    {}

public:
    /// const_iterator for the listview_sequence
    class const_iterator
        : public stlsoft_ns_qual(iterator_base)<winstl_ns_qual_std(random_access_iterator_tag)
                                            ,   sequence_value_type
                                            ,   ws_ptrdiff_t
                                            ,   void
                                            ,   sequence_value_type
                                            >
    {
        typedef const_iterator                  class_type;
    public:
        typedef sequence_value_type             value_type;

    public:
        const_iterator()
            : m_hwndListView(NULL)
            , m_index(-1)
        {}
        const_iterator(HWND hwndListView, int iIndex)
            : m_hwndListView(hwndListView)
            , m_index(iIndex)
        {}
        const_iterator(class_type const &rhs)
            : m_hwndListView(rhs.m_hwndListView)
            , m_index(rhs.m_index)
        {}

        class_type &operator =(class_type const &rhs)
        {
            m_hwndListView  =   rhs.m_hwndListView;
            m_index         =   rhs.m_index;

            return *this;
        }

    public:
        value_type operator *() const
        {
            return value_type(m_hwndListView, m_index);
        }

        bool operator ==(class_type const &rhs) const
        {
            WINSTL_MESSAGE_ASSERT("Comparing iterators from different listview_sequence instances!", m_hwndListView == rhs.m_hwndListView);

            return m_index == rhs.m_index;
        }

        bool operator !=(class_type const &rhs) const
        {
            WINSTL_MESSAGE_ASSERT("Comparing iterators from different listview_sequence instances!", m_hwndListView == rhs.m_hwndListView);

            return m_index != rhs.m_index;
        }

        class_type &operator ++()
        {
            WINSTL_MESSAGE_ASSERT("Attempting to increment an off-the-end iterator", m_index < ListView_GetItemCount(m_hwndListView));

            ++m_index;

            return *this;
        }

        /// Post-increment operator
        class_type operator ++(int)
        {
            class_type  ret(*this);

            operator ++();

            return ret;
        }

        class_type &operator --()
        {
            WINSTL_MESSAGE_ASSERT("Attempting to decrement an iterator at the start of the sequence", 0 < m_index);

            --m_index;

            return *this;
        }

        /// Post-decrement operator
        class_type operator --(int)
        {
            class_type  ret(*this);

            operator --();

            return ret;
        }

        // Random access operations

        /// Offset
        class_type &operator +=(difference_type index)
        {
            m_index += index;

            return *this;
        }

        /// Offset
        class_type &operator -=(difference_type index)
        {
            m_index -= index;

            return *this;
        }

        /// Indexing operator
        value_type operator [](difference_type index) const
        {
            return value_type(m_hwndListView, m_index + index);
        }

        /// Calculate the distance between \c this and \c rhs
        difference_type distance(class_type const &rhs) const
        {
            return m_index - rhs.m_index;
        }

        /// Pointer subtraction
        class_type operator -(difference_type n) const
        {
            return class_type(*this) -= n;
        }

        /// Pointer addition
        class_type operator +(difference_type n) const
        {
            return class_type(*this) += n;
        }

        /// Pointer difference
        difference_type operator -(class_type const &rhs) const
        {
            return distance(rhs);
        }

    // Members
    private:
        HWND    m_hwndListView;
        int     m_index;
    };

#if defined(__STLSOFT_CF_BIDIRECTIONAL_ITERATOR_SUPPORT)
    /// The non-mutating (const) reverse iterator type
    typedef stlsoft_ns_qual(const_reverse_iterator_base)<   const_iterator
                                                        ,   value_type
                                                        ,   value_type // Return by value!
                                                        ,   void*
                                                        ,   difference_type
                                                        >   const_reverse_iterator;
#endif /* __STLSOFT_CF_BIDIRECTIONAL_ITERATOR_SUPPORT */

// State
public:
    /// Returns the number of elements in the list-box
    size_type size() const
    {
        return static_cast<size_type>(ListView_GetItemCount(m_hwndListView));
    }
    /// Indicates whether the list-box is empty
    ws_bool_t empty() const
    {
        return 0 == size();
    }
    /// Returns the maximum number of items that the list-box can contain
    static size_type max_size()
    {
        return static_cast<size_type>(-1) / sizeof(LPCTSTR);
    }

// Iteration
public:
    const_iterator  begin() const
    {
        return const_iterator(m_hwndListView, 0);
    }
    const_iterator  end() const
    {
        return const_iterator(m_hwndListView, static_cast<int>(size()));
    }

#if defined(__STLSOFT_CF_BIDIRECTIONAL_ITERATOR_SUPPORT)
    /// Begins the reverse iteration
    ///
    /// \return An iterator representing the start of the reverse sequence
    const_reverse_iterator rbegin() const
    {
        return const_reverse_iterator(end());
    }
    /// Ends the reverse iteration
    ///
    /// \return An iterator representing the end of the reverse sequence
    const_reverse_iterator rend() const
    {
        return const_reverse_iterator(begin());
    }
#endif /* __STLSOFT_CF_BIDIRECTIONAL_ITERATOR_SUPPORT */

// Accessors
public:
    value_type operator [](size_type index) const
    {
        return value_type(m_hwndListView, static_cast<int>(index));
    }

private:
    HWND    m_hwndListView;
};

/* /////////////////////////////////////////////////////////////////////////////
 * Backwards compatiblity typedefs
 */

#if 0
typedef listview_sequence<lvs_string_t>         listview_sequence_old;
typedef listview_sequence_item<lvs_string_t>    listview_sequence_item_old;
#endif /* 0 */

/* /////////////////////////////////////////////////////////////////////////////
 * Shims
 */

#if 0
template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        >
inline C const *c_str_ptr_null(basic_current_directory<C, T> const &lvi)
{
    return stlsoft_ns_qual(c_str_ptr_null)(lvi.c_str());
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        >
inline C const *c_str_ptr(basic_current_directory<C, T> const &lvi)
{
    return lvi.c_str();
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        >
inline C const *c_str_data(basic_current_directory<C, T> const &lvi)
{
    return lvi.c_str();
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        >
inline ws_size_t c_str_len(basic_current_directory<C, T> const &lvi)
{
    return stlsoft_ns_qual(c_str_len)(lvi.c_str());
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        >
inline ws_size_t c_str_size(basic_current_directory<C, T> const &lvi)
{
    return stlsoft_ns_qual(c_str_size)(lvi.c_str());
}

template<   ss_typename_param_k S
        ,   ss_typename_param_k C
        ,   ss_typename_param_k T
        >
inline S &operator <<(S & s, basic_current_directory<C, T> const &lvi)
{
    s << lvi.c_str();

    return s;
}
#endif /* 0 */


inline stlsoft_ns_qual(basic_shim_string)<TCHAR, 64, true, processheap_allocator<TCHAR> > c_str_ptr_null(listview_sequence_item const &lvi)
{
    return stlsoft_ns_qual(c_str_ptr_null)(lvi.text());
}

inline stlsoft_ns_qual(basic_shim_string)<TCHAR, 64, false, processheap_allocator<TCHAR> > c_str_ptr(listview_sequence_item const &lvi)
{
    return stlsoft_ns_qual(c_str_ptr)(lvi.text());
}

inline stlsoft_ns_qual(basic_shim_string)<TCHAR, 64, false, processheap_allocator<TCHAR> > c_str_data(listview_sequence_item const &lvi)
{
    return stlsoft_ns_qual(c_str_data)(lvi.text());
}

inline ws_size_t c_str_len(listview_sequence_item const &lvi)
{
    return stlsoft_ns_qual(c_str_len)(lvi.text());
}

inline ws_size_t c_str_size(listview_sequence_item const &lvi)
{
    return stlsoft_ns_qual(c_str_size)(lvi.text());
}

template<   ss_typename_param_k S
        >
inline S &operator <<(S & s, listview_sequence_item const &lvi)
{
    s << lvi.text();

    return s;
}

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group winstl_control_library

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
# if defined(_STLSOFT_NO_NAMESPACE) || \
     defined(STLSOFT_DOCUMENTATION_SKIP_SECTION)
} // namespace winstl
# else
} // namespace winstl_project
} // namespace stlsoft
# endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 *
 * The string access shims exist either in the stlsoft namespace, or in the
 * global namespace. This is required by the lookup rules.
 *
 */

#ifndef _WINSTL_NO_NAMESPACE
# if !defined(_STLSOFT_NO_NAMESPACE) && \
     !defined(STLSOFT_DOCUMENTATION_SKIP_SECTION)
namespace stlsoft
{
# else /* ? _STLSOFT_NO_NAMESPACE */
/* There is no stlsoft namespace, so must define in the global namespace */
# endif /* !_STLSOFT_NO_NAMESPACE */

using ::winstl::c_str_ptr_null;

using ::winstl::c_str_ptr;

using ::winstl::c_str_data;

using ::winstl::c_str_len;

using ::winstl::c_str_size;

# if !defined(_STLSOFT_NO_NAMESPACE) && \
     !defined(STLSOFT_DOCUMENTATION_SKIP_SECTION)
} // namespace stlsoft
# else /* ? _STLSOFT_NO_NAMESPACE */
/* There is no stlsoft namespace, so must define in the global namespace */
# endif /* !_STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* WINSTL_INCL_WINSTL_HPP_LISTVIEW_SEQUENCE */

/* ////////////////////////////////////////////////////////////////////////// */
